#region usings
using System;
using System.ComponentModel.Composition;

using Microsoft.Kinect.Tools;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "Client", Category = "Kinect2Studio", Help = "Wrapper for KStudioClient class", Tags = "kinect2, kinect studio", Author = "ravazquez")]
	#endregion PluginInfo
	public class Kinect2StudioClientNode : IPluginEvaluate
	{
		#region fields & pins
		[Input("Enabled", IsSingle = true)]
		public ISpread<bool> FInputEnabled;
		
		[Input("Reset", IsSingle = true)]
		public ISpread<bool> FInputReset;

		[Output("Client", IsSingle = true)]
		public ISpread<KStudioClient> FOutputClient;
		
		[Output("Status", IsSingle = true)]
		public ISpread<string> FOutputStatus;
		
		private KStudioClient client;
		private string status;
		#endregion fields & pins
		
		private void InitializeClient()
		{
			KStudioClientFlags flags = KStudioClientFlags.ProcessNotifications;
			if (client == null)
			{
				client = KStudio.CreateClient(flags);
			}
			//avoid reseting the client while it is being used
			else if ((FInputReset.SliceCount > 0 && FInputReset[0]) && !client.IsServiceConnected)
			{
				client = null;
				client = KStudio.CreateClient(flags);
			}
		}

		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			if(FInputEnabled.SliceCount > 0 && !FInputEnabled[0])
			{
				client = null;
				status = "Disabled";
			}
			else if (client == null || FInputReset[0])
			{
				InitializeClient();
			}
			
			FOutputClient[0] = client;
			
			if (client != null){
				if (client.IsServiceConnected)
				{
					status = "Service is connected";
				}
				else
				{
					status = "Service is not connected";
				}
			}
			
			FOutputStatus[0] = status;
		}
	}
}
